
import uuid

import unittest

# from fom.db import FluidDB
# from fom.session import Fluid
from fom.db import PRIMITIVE_CONTENT_TYPE
from fom.dev import sandbox_fluid
from fom.mapping import (path_split, path_child, Namespace, Tag, Object,
    tag_relation, tag_value)



class TestPath(unittest.TestCase):

    def test_parent(self):
        # print path_split(u'goo/moo')
        assert path_split(u'goo/moo') == [u'goo', u'moo']

    def test_child(self):
        assert path_child(u'foo', u'blah') == u'foo/blah'

    def test_empty(self):
        assert path_child(u'', 'foo') == u'foo'


class NamespaceTest(unittest.TestCase):

    def setUp(self):
        self.fluid = sandbox_fluid()
        self.uid = str(uuid.uuid4())
        self.path = u'test/%s' % self.uid

    def test_new(self):
        n = Namespace(u'test')
        assert n.path == u'test'
        assert n.fluid is self.fluid

    def test_create(self):
        ns = Namespace(self.path)

        ns.create(u'fomtest description')

        childns = ns.create_namespace(u'child', u'fomtest child')
        childns.delete()

        # access the namespace's low-level api
        ns.api.get()
        ns.api.put(u'A better description')
        ns.api.post(u'child', u'fomtest child')

        ns.delete()

    def test_description(self):
        ns = Namespace(self.path)
        ns.create(u'testdesc')
        assert ns.description == u'testdesc'
        ns.description = u'newdesc'
        assert ns.description == u'newdesc'

    def test_create_child(self):
        n = Namespace(u'test')
        child = n.create_namespace(self.uid, u'fomtest description')
        assert child.path == self.path
        # probably should leave this, but just to be polite
        child.delete()

    def test_child_namespace(self):
        n = Namespace(u'test')
        child = n.namespace(u'test')
        assert isinstance(child, Namespace)
        assert child.path == u'test/test'

    def test_child_tag(self):
        n = Namespace(u'test')
        child = n.tag(u'test')
        assert isinstance(child, Tag)
        assert child.path == u'test/test'


class ObjectTest(unittest.TestCase):

    def setUp(self):
        self.fluid = sandbox_fluid()

    def testNew(self):
        o = Object()
        assert o.path is None
        assert o.uid is None

    def testCreate(self):
        o = Object()
        o.create()
        assert o.uid

    def testGet(self):
        o = Object()
        o.create()
        assert o.get('test/fomtest') is None

    def testSetPrimitive(self):
        o = Object()
        o.create()
        assert o.get('test/fomtest') is None
        for value in (None, True, False, 123, 45.67, 'hey there'):
            o.set('test/fomtest', value)
            self.assertEqual(o.get('test/fomtest'),
                (value, PRIMITIVE_CONTENT_TYPE))

    def testSetOpaque(self):
        o = Object()
        o.create()
        assert o.get('test/fomtest') is None
        o.set('test/fomtest', 'xyz', 'application/bananas')
        self.assertEqual(o.get('test/fomtest'), ('xyz', 'application/bananas'))

    def testHas(self):
        o = Object()
        o.create()
        assert not o.has('test/fomtest')
        o.set('test/fomtest', 123)
        assert o.has('test/fomtest')

    def testTags(self):
        o = Object()
        o.create()
        assert o.tag_paths == []
        assert o.tags == []
        o.set('test/fomtest', 123)
        assert 'test/fomtest' in o.tag_paths
        assert o.tags[0].path == 'test/fomtest'


class TagTest(unittest.TestCase):

    def setUp(self):
        self.fluid = sandbox_fluid()

    def testNew(self):
        t = Tag('test/fomtest')
        assert t.path == 'test/fomtest'

    def testGetDescription(self):
        t = Tag('test/fomtest')
        t.description = u'banana'
        assert t.description == u'banana'

    def testSetDescription(self):
        t = Tag('test/fomtest')
        t.description = u'melon'
        assert t.description == u'melon'


class RelationTest(unittest.TestCase):

    def setUp(self):
        self.fluid = sandbox_fluid()

        n = Namespace(u'test')
        child = n.create_tag(u'fomtest', u'fomtest description', False)
        child = n.create_tag(u'fomtest2', u'fomtest 2 description', False)

        class A(Object):
            fomtest = tag_relation(u'test/fomtest')
            fomtest2 = tag_value(u'test/fomtest')

        self.A = A

    def testRelation(self):
        a1 = self.A()
        a1.create()

        a2 = self.A()
        a2.create()

        a1.fomtest = a2

        self.assertEqual(a1.fomtest.uid, a2.uid)

    def testMissingValue(self):
        a1 = self.A()
        a1.create()
        assert a1.fomtest2 is None

    def testValue(self):
        a1 = self.A()
        a1.create()
        a1.fomtest2 = 'hello'
        assert a1.fomtest2 == 'hello'

